package ir.ac.ut.ece.naeem;
/*
 * Created on Mar 1, 2004
 */

/**
 * This is an abstract class that has list functions
 * @author Naeem
 * @version 1.0
 * @since insert, min, max, search, delete
 */
public abstract class List {
	/**
	 * This method is used for inserting element into the list
	 * @param key The key of the inserted element
	 */
	public abstract void insert( Comparable key);
	
	/**
	 * This method is used for finding the minimum element in the list
	 * @return The minimum element
	 */
	public abstract Item min();
	
	/**
	 * This method is used for finding the maximum element in the list
	 * @return The maximum element
	 */
	public abstract Item max();
	
	/**
	 * This method is used for finding one element in the list
	 * This metod ease writing delete and search
	 * @see DItem
	 * @see List#search(Comparable)
	 * @see List#delete(Comparable)
	 * @param key The key of the specific item
	 * @return The first element in DItem is matched item and the second previous one
	 */
	protected abstract DItem find( Comparable key);

	/**
	 * This method is used for finding one element in the list
	 * @see List#find(Comparable)
	 * @param key The key of the specific item
	 * @return The maximum element
	 */
	public Item search( Comparable key) {
		Item found = find( key).first;

		if( found == null || !found.key.equals( key))
			return null;
			
		return found;
	}
	
	/**
	 *  This method is used for deleting one element in the list
	 * @see List#find(Comparable)
	 * @param key The key of the specific item
	 * @throws ItemNotFound Tells that you try to delete nothing!
	 */
	public void delete( Comparable key) throws ItemNotFound{
		DItem temp = find( key);
		Item found = temp.first;
		Item previous = temp.second;
		
		if( found != null && found.key.equals( key)) {
			if( previous != null)
				previous.item = found.item;
			else
				head = found.item;
		}else
			throw new ItemNotFound();
	}
	
	/**
	 * The head of the list
	 * @see Item
	 */
	protected Item head;
}